# -*- coding: UTF-8 -*-

bl_info = {
    "name": "GraphKit",
    "description": "用于在曲线图编辑器中操纵F曲线和关键帧的工具",
    "author": "Blastframe",
    "version": (2, 0, 0),
    "blender": (4, 2, 0),
    "location": "Graph Editor > Sidebar > GraphKit Panel",
    "doc_url": "https://blastframe.com/product/graphkit/",
    "tracker_url": "",
    "category": "Animation"
}

if "bpy" in locals():
    import importlib

    if "tween" in locals():
        importlib.reload(tween)
    if "range_to_loop" in locals():
        importlib.reload(range_to_loop)
    if "perfect_loop" in locals():
        importlib.reload(perfect_loop)
    if "repeat" in locals():
        importlib.reload(repeat)
    if "offset" in locals():
        importlib.reload(offset)
    if "distribute" in locals():
        importlib.reload(distribute)
    if "fade" in locals():
        importlib.reload(fade)
    if "flatten_holds" in locals():
        importlib.reload(flatten_holds)
    if "amplify" in locals():
        importlib.reload(amplify)
    if "randomize" in locals():
        importlib.reload(randomize)

    if "preferences" in locals():
        importlib.reload(preferences)
    if "utils" in locals():
        importlib.reload(utils)
    if "check_updates" in locals():
        importlib.reload(check_updates)
    if "panel" in locals():
        importlib.reload(panel)

import bpy

from .operators import (
    offset,
    range_to_loop,
    perfect_loop,
    repeat,
    distribute,
    fade,
    flatten_holds,
    amplify,
    randomize,
    tween
)

from .ui import (
    panel
)

from .utils import (
    check_updates,
    utils
)

from .utils.utils import load_icons
from . import preferences

preview_collections = {}
class GRAPH_MT_GraphKit_Menu(bpy.types.Menu):
    bl_idname = 'GRAPH_MT_GraphKit_Menu'
    bl_label = "GraphKit"
    bl_description = "GraphKit Menu"

    def draw(self, context):
        global preview_collections
        pcoll = preview_collections["main"]

        amplify_icon = pcoll["amplify_icon"]
        distribute_icon = pcoll["distribute_icon"]
        fade_icon = pcoll["fade_icon"]
        flatten_holds_icon = pcoll["flatten_holds_icon"]
        randomize_icon = pcoll["randomize_icon"]
        repeat_icon = pcoll["repeat_icon"]

        layout = self.layout

        layout.operator("graph.amplify", icon_value=amplify_icon.icon_id)
        layout.operator("graph.distribute", icon_value=distribute_icon.icon_id)
        layout.operator("graph.fade", icon_value=fade_icon.icon_id)
        layout.operator("graph.flatten_holds", icon_value=flatten_holds_icon.icon_id)
        layout.operator("graph.offset", icon="ARROW_LEFTRIGHT")
        layout.operator("graph.perfect_loop", icon="FILE_REFRESH")
        layout.operator("graph.randomize", icon_value=randomize_icon.icon_id)
        layout.operator("graph.range_to_loop", icon="PREVIEW_RANGE")
        layout.operator("graph.repeat", icon_value=repeat_icon.icon_id)

def menu_func(self, context):
    self.layout.separator()
    self.layout.menu(GRAPH_MT_GraphKit_Menu.bl_idname)

def register():
    global preview_collections
    preview_collections = load_icons()

    preferences.register()
    check_updates.register()
    tween.register()
    range_to_loop.register()
    perfect_loop.register()
    repeat.register()
    offset.register()
    fade.register()
    flatten_holds.register()
    distribute.register()
    amplify.register()
    randomize.register()
    panel.register()
    bpy.utils.register_class(GRAPH_MT_GraphKit_Menu)
    bpy.types.GRAPH_MT_key.append(menu_func)

def unregister():
    global preview_collections
    bpy.types.GRAPH_MT_key.remove(menu_func)
    bpy.utils.unregister_class(GRAPH_MT_GraphKit_Menu)

    panel.unregister()
    tween.unregister()
    fade.unregister()
    flatten_holds.unregister()
    distribute.unregister()
    offset.unregister()
    repeat.unregister()
    perfect_loop.unregister()
    range_to_loop.unregister()
    randomize.unregister()
    amplify.unregister()
    check_updates.unregister()
    preferences.unregister()

    for pcoll in preview_collections.values():
        bpy.utils.previews.remove(pcoll)
    preview_collections.clear()