import bpy, os, math
from ..utils.utils import (
    refresh,
    get_fcurve_selected_keys
)
from bpy.types import Operator

DEBUG_MODE = False
class GRAPH_OT_Flatten_Holds(Operator):
    """Flatten hold keyframes"""
    bl_idname = "graph.flatten_holds"
    bl_label = "Flatten Holds"
    bl_options = { "REGISTER", "UNDO" }

    flatten_holds_type: bpy.props.EnumProperty(
        name= "Type",
        default="SELECTED_FCURVE",
        description= "Selection to use for flattening holds.",
        items= [("SELECTED_FCURVE","Selected F-Curves","Flatten holds of all keyframes of selected F-Curves."),
                ("SELECTED_KEYS","Selected Keyframes","Flatten holds of selected keyframes.")
                ]
    )

    threshold: bpy.props.FloatProperty(
        name= "Threshold",
        step=0.001,
        default=0.1,
        description= "Number of frames/notes by which to distribute",
        soft_min = 0.001, soft_max = 30.0
    )

    def flatten_holds(self,keyframes):
        for i,keyframe in enumerate(keyframes):
            """
            if i == 0:
                continue
            """
            if i == len(keyframes)-1:
                break
            first_element = keyframe.co[1]
            next_element = keyframes[i+1].co[1]
            difference = next_element-first_element
            if DEBUG_MODE == True:
                print("difference",difference)
            if abs(difference) < self.threshold:
                keyframe.handle_left[1] = first_element
                keyframe.handle_right[1] = first_element
                keyframes[i+1].co[1] = first_element
                keyframes[i+1].handle_left[1] = keyframes[i+1].co[1]
                keyframes[i+1].handle_right[1] = keyframes[i+1].co[1]

    def execute(self, context):
        scene = context.scene
        fcurves = bpy.context.selected_editable_fcurves[:]
        for fcurve in fcurves:
            if self.flatten_holds_type == "SELECTED_FCURVE":
                self.flatten_holds(fcurve.keyframe_points)
            elif self.flatten_holds_type == "SELECTED_KEYS":
                self.flatten_holds(get_fcurve_selected_keys(fcurve))

        refresh()
        return {'FINISHED'}

def register():
    bpy.utils.register_class(GRAPH_OT_Flatten_Holds)

def unregister():
    bpy.utils.unregister_class(GRAPH_OT_Flatten_Holds)
