import bpy
from ..utils.utils import refresh_ui_keyframes, get_keyframe, get_keyframe_index

DEBUG_MODE = False

def add_init_values():
    fcurves = bpy.context.selected_editable_fcurves[:]
    bpy.types.Scene.tween_values.clear()
    current_frame = bpy.data.scenes['Scene'].frame_current
    for fcurve in fcurves:
        bpy.types.Scene.tween_values.append({"fcurve":fcurve,"value":fcurve.evaluate(current_frame)})
    if DEBUG_MODE: print("add_init_values",bpy.types.Scene.tween_values)

class DRAGGABLEPROP_OT_subscribe(bpy.types.Operator):
    bl_idname  = "draggableprop.subscribe"
    bl_label   = ""
    stop: bpy.props.BoolProperty()  # This is used so we don't end up in an infinite loop because we blocked the release event

    def modal(self, context, event):
        if self.stop:
            context.scene.tween_props.is_dragging = False
            if DEBUG_MODE: print("End Dragging !")
            self.stop = True
            return {'FINISHED'}
        if event.value == 'RELEASE':  # Stop the modal on next frame. Don't block the event since we want to exit the field dragging
            if DEBUG_MODE: print("RELEASE")
            context.scene.tween_props.is_dragging = False
            self.stop = True
            bpy.types.Scene.tween_values.clear()
        return {'PASS_THROUGH'}

    def invoke(self, context, event):
        if DEBUG_MODE: print("invoke")
        fcurves = bpy.context.selected_editable_fcurves[:]
        bpy.types.Scene.tween_values.clear()
        current_frame = bpy.data.scenes['Scene'].frame_current
        for fcurve in fcurves:
            bpy.types.Scene.tween_values.append({"fcurve":fcurve,"value":fcurve.evaluate(current_frame)})
        self.stop = False
        context.window_manager.modal_handler_add(self)
        return {'RUNNING_MODAL'}

def set_track_values(self,tween_amount):
    context = bpy.context

    if len(bpy.types.Scene.tween_values) == 0:
        add_init_values()

    #fcurves = bpy.context.selected_editable_fcurves[:]
    fcurves = [f['fcurve'] for f in bpy.types.Scene.tween_values]
    init_values = [v['value'] for v in bpy.types.Scene.tween_values]

    #if DEBUG_MODE: print("init_values",init_values)

    for i,fcurve in enumerate(fcurves):
        key_count = len(fcurve.keyframe_points)
        current_frame = bpy.context.scene.frame_current
        current_value = fcurve.evaluate(current_frame)
        found = get_keyframe(fcurve,current_frame)
        keyIndex = None
        key = None

        if found:
            key = found
        else:
            key = fcurve.keyframe_points.insert(current_frame, current_value)

        keyIndex = get_keyframe_index(fcurve,current_frame)

        inter = "BEZIER"

        prev_distance = None
        try:
            prevKey = fcurve.keyframe_points[keyIndex-1]
            inter = fcurve.keyframe_points[keyIndex-1].interpolation
            prev_distance = prevKey.co[1]-init_values[i]
        except IndexError:
            continue

        next_distance = None
        try:
            nextKey = fcurve.keyframe_points[keyIndex+1]
            next_distance = nextKey.co[1]-init_values[i]
            if DEBUG_MODE: print("prev_distance",prev_distance,"next_distance",next_distance)
        except IndexError:
            continue

        if prev_distance == 0:
            prev_distance = nextKey.co[1]-prevKey.co[0]
        if next_distance == 0:
            next_distance = nextKey.co[1]-prevKey.co[0]

        newValue = init_values[i]
        if tween_amount > 0:
            newValue = init_values[i] + (next_distance*tween_amount)
        elif tween_amount < 0:
            newValue = init_values[i] + (prev_distance*abs(tween_amount))

        #newValue = prevKey.co[1]+difference*tween_amount
        key.interpolation = inter
        if inter == "BEZIER":
            key.handle_left_type = "AUTO_CLAMPED"
            key.handle_right_type = "AUTO_CLAMPED"
        key.type = "BREAKDOWN"
        key.co[1] = newValue

        fcurve.update()
    refresh_ui_keyframes()

def update_prop(self, value):
    if DEBUG_MODE: print("self.is_dragging",self.is_dragging)
    if self.is_dragging == True:
        #if DEBUG_MODE: print("Dragging the slider !")
        pass
    else:
        if DEBUG_MODE: print("Beginning dragging the slider !")
        self.is_dragging = True
        bpy.ops.draggableprop.subscribe('INVOKE_DEFAULT')

class Tween_Props(bpy.types.PropertyGroup):
    tween_amt: bpy.props.FloatProperty(
        name = "Tween",
        description = "Create a keyframe between two keyframes that by a "\
        "percentage of the difference",
        set=set_track_values,
        update=update_prop,
        soft_min = -2, soft_max = 2
    )
    is_dragging: bpy.props.BoolProperty(
        default=False
    )

def register():
    bpy.utils.register_class(DRAGGABLEPROP_OT_subscribe)  # Register modal operator
    bpy.utils.register_class(Tween_Props)
    bpy.types.Scene.tween_values = list()
    bpy.types.Scene.tween_props = bpy.props.PointerProperty(type= Tween_Props)

def unregister():
    bpy.utils.unregister_class(DRAGGABLEPROP_OT_subscribe)
    del bpy.types.Scene.tween_props
    bpy.utils.unregister_class(Tween_Props)
    del bpy.types.Scene.tween_values
