import bpy, os
import bpy.utils.previews as previews

class GRAPHKIT_preferences(bpy.types.AddonPreferences):
    bl_idname = __package__
    update: bpy.props.BoolProperty(name="Update", default=False)
    version: bpy.props.StringProperty(name="Version", default="")

    def draw(self, context):
        pcoll = preview_collections["main"]
        blastframe_logo = pcoll["blastframe_logo"]
        blender_market_logo = pcoll["blender_market_logo"]
        update_icon = pcoll["update"]
        layout = self.layout
        col = layout.column()

        col.operator("graphkit.check_updates", text="Check For Updates", icon_value=update_icon.icon_id)
        if self.update:
            col.alert = True
            col.label(text="Update Available: %s. Please download from the store where you purchased this Add-on."%(str(self.version)))
        elif self.version != "":
            col.label(text="You have the latest version.")
        col.alert = False
        col.operator("wm.url_open", text="Blastframe", icon_value=blastframe_logo.icon_id).url = "https://blastframe.com"
        col.operator("wm.url_open", text="Blender Market", icon_value=blender_market_logo.icon_id).url = "https://blendermarket.com/creators/blastframe"

preview_collections = {}

def register():
    bpy.utils.register_class(GRAPHKIT_preferences)
    pcoll = previews.new()
    icon_dir = os.path.join(os.path.dirname(__file__), "icons")
    pcoll.load("blender_market_logo", os.path.join(icon_dir, "Blender_Market.png"), 'IMAGE')
    pcoll.load("blastframe_logo", os.path.join(icon_dir, "Blastframe_Logo_White.png"), 'IMAGE')
    pcoll.load("update", os.path.join(icon_dir, "update.png"), 'IMAGE')
    preview_collections["main"] = pcoll

def unregister():
    bpy.utils.unregister_class(GRAPHKIT_preferences)
    for pcoll in preview_collections.values():
        previews.remove(pcoll)
    preview_collections.clear()
