import bpy, os
from pathlib import Path
from bpy.types import Panel
from ..utils.utils import load_icons

preview_collections = {}
class GRAPH_PT_GraphKit_Panel(Panel):
    """Creates a Panel in the Graph Editor context"""
    bl_label = ""
    bl_category = "GraphKit"
    bl_idname = "GRAPH_PT_GraphKit_Panel"
    bl_space_type = "GRAPH_EDITOR"
    bl_region_type = "UI"
    bl_context = "scene"

    def draw_header(self, context):
        self.layout.label(text="GraphKit", icon="TOOL_SETTINGS")

    def draw(self, context):
        layout = self.layout
        scene = context.scene
        tween_props = scene.tween_props

        layout.separator()

        pcoll = preview_collections["main"]
        amplify_icon = pcoll["amplify_icon"]
        fade_icon = pcoll["fade_icon"]
        flatten_holds_icon = pcoll["flatten_holds_icon"]
        distribute_icon = pcoll["distribute_icon"]
        randomize_icon = pcoll["randomize_icon"]
        repeat_icon = pcoll["repeat_icon"]

        col = layout.column(align=True)
        col.prop( tween_props, 'tween_amt', slider=True )

        layout.use_property_split = True
        layout.use_property_decorate = False

        layout.separator()

        col = layout.column(align=True)
        col.operator("graph.amplify", icon_value=amplify_icon.icon_id)
        col.operator("graph.distribute", icon_value=distribute_icon.icon_id)
        col.operator("graph.fade", icon_value=fade_icon.icon_id)
        col.operator("graph.flatten_holds", icon_value=flatten_holds_icon.icon_id)
        col.operator("graph.offset", icon="ARROW_LEFTRIGHT")
        col.operator("graph.perfect_loop", icon="FILE_REFRESH")
        col.operator("graph.randomize", icon_value=randomize_icon.icon_id)
        col.operator("graph.range_to_loop", icon="PREVIEW_RANGE")
        col.operator("graph.repeat", icon_value=repeat_icon.icon_id)

def register():
    global preview_collections
    preview_collections = load_icons()
    bpy.utils.register_class(GRAPH_PT_GraphKit_Panel)

def unregister():
    global preview_collections
    bpy.utils.unregister_class(GRAPH_PT_GraphKit_Panel)

    for pcoll in preview_collections.values():
        bpy.utils.previews.remove(pcoll)
    preview_collections.clear()

if __name__ == "__main__":
    bpy.context.scene.tween_props.is_dragging = False
