import bpy, math, os
from collections import Counter
from pathlib import Path

def refresh():
    #Updates all animation related UI
    for area in bpy.context.screen.areas:
        if area.type == 'VIEW_3D':
            for region in area.regions:
                region.tag_redraw()
        elif area.type == 'DOPESHEET_EDITOR':
            for region in area.regions:
                region.tag_redraw()
        elif area.type == 'GRAPH_EDITOR':
            for region in area.regions:
                region.tag_redraw()
        elif area.type == 'NLA_EDITOR':
            for region in area.regions:
                region.tag_redraw()
        elif area.type == 'OUTLINER':
            for region in area.regions:
                region.tag_redraw()
        elif area.type == 'PROPERTIES':
            for region in area.regions:
                region.tag_redraw()

def load_icons():
    preview_collections = {}
    icons = [
        {"name":"amplify_icon","file":"Amplify.png"},
        {"name":"distribute_icon","file":"Distribute.png"},
        {"name":"fade_icon","file":"Fade.png"},
        {"name":"flatten_holds_icon","file":"Flatten_Holds.png"},
        {"name":"randomize_icon","file":"Randomize.png"},
        {"name":"repeat_icon","file":"Repeat.png"}
    ]

    pcoll = bpy.utils.previews.new()
    path = Path(os.path.dirname(__file__))
    parent = path.parent.absolute()
    icon_dir = os.path.join(parent, "icons")

    for i,icon in enumerate(icons):
        icon_path = os.path.join(icon_dir, icon["file"])
        pcoll.load(icon["name"], icon_path, 'IMAGE')

    preview_collections["main"] = pcoll
    return preview_collections

def update_fcurves():
    for fcurve in bpy.context.visible_fcurves[:]:
        for keyframe in fcurve.keyframe_points:
            if keyframe.select_control_point:
                fcurve.update()
                break

def get_fcurve_selected_keys(fcurve, no_limits=False):
    selected_keys = []

    for keyframe in fcurve.keyframe_points:
        if keyframe.select_control_point:
            selected_keys.append(keyframe)

    if no_limits == True:
        selected_keys = selected_keys[1:-1]

    return selected_keys

def get_all_selected_keys():
    selected_keys = []

    fcurves = bpy.context.selected_editable_fcurves[:]
    for fcurve in fcurves:
        #print(fcurve.id_data)
        for keyframe in fcurve.keyframe_points:
            if keyframe.select_control_point:
                selected_keys.append(keyframe)

    return selected_keys

def get_selected_keys(fcurve,mode,exclude_limits=False):
    selection = list()

    if mode == "SELECTED_FCURVE":
        selection = fcurve.keyframe_points

    elif mode == "SELECTED_KEYS":
        selection = get_fcurve_selected_keys(fcurve)

    if exclude_limits == True:
        selection = selection[1:-1]

    return selection


def get_keyframe(fcurve,frame):
    found = None
    for keyframe in fcurve.keyframe_points:
        if keyframe.co[0] == frame:
            found = keyframe
            break
    return found

def get_keyframe_index(fcurve,frame):
    foundIndex = None
    for i,keyframe in enumerate(fcurve.keyframe_points):
        if keyframe.co[0] == frame:
            foundIndex = i
            break
    return foundIndex

def refresh_ui_keyframes():
    try:
        for area in bpy.context.screen.areas:
            if area.type in ('TIMELINE', 'GRAPH_EDITOR', 'DOPESHEET_EDITOR'):
                area.tag_redraw()
    except:
        pass

def get_linear_x_value(x1,y1,x2,y2,y_value):
    line_slope = (y1-y2)/(x1-x2)
    yintercept = (x1*y2 - x2*y1)/(x1-x2)
    try:
        frame = (y_value - yintercept)/ line_slope
        return frame
    except ZeroDivisionError:
        return 0

def get_linear_y_value(x1,y1,x2,y2,frame):
    line_slope = (y1-y2)/(x1-x2)
    yintercept = (x1*y2 - x2*y1)/(x1-x2)
    value = line_slope * frame + yintercept
    return value

def convert_to_auto(fcurves,exclude_start_and_end):
    for fcurve in fcurves:
        selected_keys = get_fcurve_selected_keys(fcurve,
            no_limits=exclude_start_and_end)
        for keyframe in selected_keys:
            keyframe.handle_left_type = "AUTO"
            keyframe.handle_right_type = "AUTO"
        fcurve.update()

def magnitude(p):
  return math.sqrt(p[0]**2 + p[1]**2)

def normalize(p):
  m = magnitude(p)
  if (m > 0):
    return [p[0]/m,p[1]/m]

def line_length(pt1,pt2):
    a = pt2[0] - pt1[0]
    b = pt2[1] - pt1[1]
    return math.sqrt(a**2 + b**2)

def dot(pt1, pt2):
    return pt1[0]*pt2[0] + pt1[1]*pt2[1]

def diff(pt1,pt2):
  return [pt1[0]-pt2[0],pt1[1]-pt2[1]]

def sqr(pt):
  return [pt[0]**2,pt[1]**2]

def prod(pt1,pt2):
  return [pt1[0]*pt2[0],pt1[1]*pt2[1]]

def mult(pt,f):
  return [pt[0]*f,pt[1]*f]

def add(pt1,pt2):
  return [pt1[0]+pt2[0],pt1[1]+pt2[1]]

def div(pt,q):
  return [pt[0]/q,pt[1]/q]

def get_handle_length(x1, x2, y1, y2):
    a = x1 - x2
    b = y1 - y2

    c = math.sqrt( a*a + b*b )
    return c

def sub_v2_v2v2(a, b):
    r = [0,0]
    r[0] = a[0] - b[0]
    r[1] = a[1] - b[1]
    return r

def dot_v2v2(a, b):
    return a[0] * b[0] + a[1] * b[1]

def mul_v2_v2fl(r, a, f):
    r[0] = a[0] * f
    r[1] = a[1] * f

def zero_v2(r):
    r[0] = 0.0
    r[1] = 0.0

def normalize_v2_v2_length(r, a, unit_length):
    d = dot_v2v2(a, a)
    if (d > 1.0e-35):
        d = math.sqrt(d)
        mul_v2_v2fl(r, a, unit_length / d)
    else:
        # Either the vector is small or one of it's values contained `nan`.
        zero_v2(r)
        d = 0.0
    return d

def normalize_v2_length(n, unit_length):
    return normalize_v2_v2_length(n, n, unit_length)

def add_v2_v2v2(r, a, b):
    r[0] = a[0] + b[0]
    r[1] = a[1] + b[1]
    return r

def most_frequent(List):
    occurence_count = Counter(List)
    return occurence_count.most_common(1)[0][0]

def align_handles(match_mode,keyframe1,keyframe2):
    key1_left_handle_direction = sub_v2_v2v2(keyframe1.handle_left, keyframe1.co)
    key1_right_handle_direction = sub_v2_v2v2(keyframe1.handle_right, keyframe1.co)
    key1_left_handle_length = get_handle_length(keyframe1.co[0],keyframe1.handle_left[0],
        keyframe1.co[1],keyframe1.handle_left[1])
    key1_right_handle_length = get_handle_length(keyframe1.co[0],keyframe1.handle_right[0],
        keyframe1.co[1],keyframe1.handle_right[1])

    #key2_left_handle_direction = sub_v2_v2v2(keyframe2.handle_left, keyframe2.co)
    #key2_right_handle_direction = sub_v2_v2v2(keyframe2.handle_right, keyframe2.co)
    key2_left_handle_length = get_handle_length(keyframe2.co[0],keyframe2.handle_left[0],
        keyframe2.co[1],keyframe2.handle_left[1])
    key2_right_handle_length = get_handle_length(keyframe2.co[0],keyframe2.handle_right[0],
        keyframe2.co[1],keyframe2.handle_right[1])

    if match_mode == "MATCH_FIRST":
        #matches left handle to right handle
        normalize_v2_length(key1_right_handle_direction,key1_left_handle_length)
        add_v2_v2v2(keyframe1.handle_left,keyframe1.co,[
            -key1_right_handle_direction[0],
            -key1_right_handle_direction[1]
        ])
        normalize_v2_length(key1_right_handle_direction,key2_left_handle_length)
        add_v2_v2v2(keyframe2.handle_left,keyframe2.co,[
            -key1_right_handle_direction[0],
            -key1_right_handle_direction[1]
        ])
        normalize_v2_length(key1_right_handle_direction,key2_right_handle_length)
        add_v2_v2v2(keyframe2.handle_right,keyframe2.co,[
            key1_right_handle_direction[0],
            key1_right_handle_direction[1]
        ])
    elif match_mode == "MATCH_LAST":
        #matches right handle to left handle
        normalize_v2_length(key1_left_handle_direction,key1_right_handle_length)
        add_v2_v2v2(keyframe1.handle_right,keyframe1.co,[
            -key1_left_handle_direction[0],
            -key1_left_handle_direction[1]
        ])

        normalize_v2_length(key1_left_handle_direction,key2_right_handle_length)
        add_v2_v2v2(keyframe2.handle_right,keyframe2.co,[
            -key1_left_handle_direction[0],
            -key1_left_handle_direction[1]
        ])

        normalize_v2_length(key1_left_handle_direction,key2_left_handle_length)
        add_v2_v2v2(keyframe2.handle_left,keyframe2.co,[
            key1_left_handle_direction[0],
            key1_left_handle_direction[1]
        ])

def match_handle_lengths(keyframe1,keyframe2):
    key1_left_handle_direction = sub_v2_v2v2(keyframe1.handle_left, keyframe1.co)
    key1_right_handle_direction = sub_v2_v2v2(keyframe1.handle_right, keyframe1.co)
    key1_left_handle_length = get_handle_length(keyframe1.co[0],keyframe1.handle_left[0],
        keyframe1.co[1],keyframe1.handle_left[1])
    key1_right_handle_length = get_handle_length(keyframe1.co[0],keyframe1.handle_right[0],
        keyframe1.co[1],keyframe1.handle_right[1])

    normalize_v2_length(key1_left_handle_direction, key1_left_handle_length)
    add_v2_v2v2(keyframe2.handle_left,keyframe2.co,key1_left_handle_direction)
    normalize_v2_length(key1_right_handle_direction, key1_right_handle_length)
    add_v2_v2v2(keyframe2.handle_right,keyframe2.co,key1_right_handle_direction)

def loop_keys(match_mode,match_angles,match_lengths,key1,key2):
    if match_mode == "MATCH_FIRST":
        key2.co[1] = key1.co[1]
        if match_lengths == True:
            match_handle_lengths(key1,key2)
        if match_angles == True:
            align_handles(match_mode,key1,key2)
    elif match_mode == "MATCH_LAST":
        key1.co[1] = key2.co[1]
        if match_lengths == True:
            match_handle_lengths(key2,key1)
        if match_angles == True:
            align_handles(match_mode,key2,key1)